#!/usr/bin/env python
""" rfkill_.py: rfkill header port

/usr/include/linux
/*
 * Copyright (C) 2006 - 2007 Ivo van Doorn
 * Copyright (C) 2007 Dmitry Torokhov
 * Copyright 2009 Johannes Berg <johannes@sipsolutions.net>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

Implements userspace program rfkill in Python to query the state of rfkill switches

NOTE:
 o rfkill_block will block all devices regardless of index, however, the blocked
  state will only shown in device that was submitted for blocking - this is the
  same behavior seen in rfkill block <idx>
  - this may be due to bug in ubuntu and not present in other distros
 o rfkill does not do sanity checks on the index, rfkill.py will through error
  if the index does not exist
  
"""

__name__ = 'rfkill_h'
__license__ = 'GPLv3'
__version__ = '0.0.1'
__date__ = 'June 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import struct

RFKILL_STATE_SOFT_BLOCKED = 0
RFKILL_STATE_UNBLOCKED    = 1
RFKILL_STATE_HARD_BLOCKED = 2

"""
/**
 * enum rfkill_type - type of rfkill switch.
 *
 * @RFKILL_TYPE_ALL: toggles all switches (requests only - not a switch type)
 * @RFKILL_TYPE_WLAN: switch is on a 802.11 wireless network device.
 * @RFKILL_TYPE_BLUETOOTH: switch is on a bluetooth device.
 * @RFKILL_TYPE_UWB: switch is on a ultra wideband device.
 * @RFKILL_TYPE_WIMAX: switch is on a WiMAX device.
 * @RFKILL_TYPE_WWAN: switch is on a wireless WAN device.
 * @RFKILL_TYPE_GPS: switch is on a GPS device.
 * @RFKILL_TYPE_FM: switch is on a FM radio device.
 * @RFKILL_TYPE_NFC: switch is on an NFC device.
 * @NUM_RFKILL_TYPES: number of defined rfkill types
 */
"""
RFKILL_TYPES = ['all','wlan','bluetooth','uwb','wimax','wwan','gps','fm','nfc']
RFKILL_TYPE_ALL       = 0
RFKILL_TYPE_WLAN      = 1
RFKILL_TYPE_BLUETOOTH = 2
RFKILL_TYPE_UWB       = 3
RFKILL_TYPE_WIMAX     = 4
RFKILL_TYPE_WWAN      = 5
RFKILL_TYPE_GPS       = 6
RFKILL_TYPE_FM        = 7
RFKILL_TYPE_NFC       = 8
NUM_RFKILL_TYPES      = 9

"""
/**
 * enum rfkill_operation - operation types
 * @RFKILL_OP_ADD: a device was added
 * @RFKILL_OP_DEL: a device was removed
 * @RFKILL_OP_CHANGE: a device's state changed -- userspace changes one device
 * @RFKILL_OP_CHANGE_ALL: userspace changes all devices (of a type, or all)
 */
 """
RFKILL_OP_ADD        = 0
RFKILL_OP_DEL        = 1
RFKILL_OP_CHANGE     = 2
RFKILL_OP_CHANGE_ALL = 3

"""
/**
 * struct rfkill_event - events for userspace on /dev/rfkill
 * @idx: index of dev rfkill
 * @type: type of the rfkill struct
 * @op: operation code
 * @hard: hard state (0/1)
 * @soft: soft state (0/1)
 *
 * Structure used for userspace communication on /dev/rfkill,
 * used for events from the kernel and control to the kernel.
 */
"""
rfk_rfkill_event = "IBBBB"
RFKILLEVENTLEN = struct.calcsize(rfk_rfkill_event)
def rfkill_event(idx,rtype,op,hard=0,soft=0):
    """
     create a rkfill event structure
     :param idx: index of dev rfkill i.e. 0,1
     :param rtype: type of rfkill
     :param op: op code
     :param hard: hard state one of {0=unbloacked|1=blocked}
     :param soft: soft state one of {0=unblocked|1=blocked}
     :returns: a rfkill event structure
    """
    return struct.pack(rfk_rfkill_event,idx,rtype,op,hard,soft)


#!/usr/bin/env python

""" hardware.py: device related utility functions

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

Defines device hardware related functions: mac address, driver, chipset etc

"""

__name__ = 'hardware'
__license__ = 'GPLv3'
__version__ = '0.0.5'
__date__ = 'June 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import os
import random

dpath = '/proc/net/dev' # system device details
drvpath = '/sys/class/net/{0}/device/driver/module/drivers' # format w/ device name

def oui(mac):
    """
     :param mac: 48-bit mac address
     :returns: oui portion of mac address
    """
    return mac[:8]

def ulm(mac):
    """
    :param mac:
    :returns: ulm portion of mac address
    """
    return mac[9:]

def manufacturer(ouis,mac):
    """
     returns the manufacturer of the mac address if exists, otherwise 'unknown'
     :param ouis: oui dict
     :param mac: hw addr to search up
     :returns: manufacturer
    """
    try:
        return ouis[mac[:8]]
    except KeyError:
        return "unknown"

def randhw(ouis=None):
    """
     generate a random hw address
     :param ouis: oui dict to use (if any)
     :returns: random hw address
    """
    if ouis is None or ouis == []:
        o = ":".join(['{0:02x}'.format(random.randint(0,255)) for _ in range(3)])
    else:
        o = random.choice(ouis.keys())
    u = ":".join(['{0:02x}'.format(random.randint(0,255)) for _ in range(3)])
    return o + ':' + u

def ifcard(dev):
    """
     get driver and chipset for dev
     :param dev: device name
     :returns: tuple t = (driver,chipset)
    """
    driver = ifdriver(dev)
    return driver,ifchipset(driver)

def ifdriver(dev):
    """
     get driver for dev
     :param dev: device name
     :returns: driver (or unknown)
    """
    try:
        # find the driver for nic in driver's module, split on ':' and return
        ds = os.listdir(drvpath.format(dev))
        if len(ds) > 1: return "Unknown"
        return ds[0].split(':')[1]
    except OSError:
        return "Unknown"

def ifchipset(driver):
    """
    returns the chipset for given driver (Thanks aircrack-ng team)
    :param driver: nic driver
    :returns: chipset of driver
    NOTE: does not fully implement the airmon-ng getChipset where identification
    requires system commands
    """
    if driver == "Unknown": return "Unknown"
    if driver == "Otus" or driver == "arusb_lnx": return "AR9001U"
    if driver == "WiLink": return "TIWLAN"
    if driver == "ath9k_htc" or driver == "usb": return "AR9001/9002/9271"
    if driver.startswith("ath") or driver == "ar9170usb": return "Atheros"
    if driver == "zd1211rw_mac80211": return "ZyDAS 1211"
    if driver == "zd1211rw": return "ZyDAS"
    if driver.startswith("acx"): return "TI ACX1xx"
    if driver == "adm8211": return "ADMtek 8211"
    if driver == "at76_usb": return "Atmel"
    if driver.startswith("b43") or driver == "bcm43xx": return "Broadcom"
    if driver.startswith("p54") or driver == "prism54": return "PrismGT"
    if driver == "hostap": return "Prism 2/2.5/3"
    if driver == "r8180" or driver == "rtl8180": return "RTL8180/RTL8185"
    if driver == "rtl8187" or driver == "r8187": return "RTL8187"
    if driver == "rt2570" or driver == "rt2500usb": return "Ralink 2570 USB"
    if driver == "rt2400" or driver == "rt2400pci": return "Ralink 2400 PCI"
    if driver == "rt2500" or driver == "rt2500pci": return "Ralink 2560 PCI"
    if driver == "rt61" or driver == "rt61pci": return "Ralink 2561 PCI"
    if driver == "rt73" or driver == "rt73usb": return "Ralink 2573 USB"
    if driver == "rt2800" or driver == "rt2800usb" or driver == "rt3070sta": return "Ralink RT2870/3070"
    if driver == "ipw2100": return "Intel 2100B"
    if driver == "ipw2200": return "Intel 2200BG/2915ABG"
    if driver == "ipw3945" or driver == "ipwraw" or driver == "iwl3945": return "Intel 3945ABG"
    if driver == "ipw4965" or driver == "iwl4965": return "Intel 4965AGN"
    if driver == "iwlagn" or driver == "iwlwifi": return "Intel 4965/5xxx/6xxx/1xxx"
    if driver == "orinoco": return "Hermes/Prism"
    if driver == "wl12xx": return "TI WL1251/WL1271"
    if driver == "r871x_usb_drv": return "Realtek 81XX"
    return "Unknown"


#!/usr/bin/env python

""" nl80211_c.py: nl80211 attribute policy definitions

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

A port of nla_policy definitions found in nl80211.c to python

"""

__name__ = 'nl80211_c'
__license__ = 'GPLv3'
__version__ = '0.0.5'
__date__ = 'July 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import pyric.net.netlink_h as nlh
import pyric.net.wireless.nl80211_h as nl80211h

#static const struct nla_policy nl80211_policy[NUM_NL80211_ATTR] = {
nl80211_policy = {
    nl80211h.NL80211_ATTR_WIPHY:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_NAME:nlh.NLA_STRING,
    nl80211h.NL80211_ATTR_WIPHY_TXQ_PARAMS:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_WIPHY_FREQ:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_CHANNEL_TYPE:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_CHANNEL_WIDTH:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_CENTER_FREQ1:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_CENTER_FREQ2:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_RETRY_SHORT:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_WIPHY_RETRY_LONG:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_WIPHY_FRAG_THRESHOLD:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_RTS_THRESHOLD:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_COVERAGE_CLASS:nlh.NLA_U8,
    #nl80211h.NL80211_ATTR_WIPHY_DYN_ACK:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_IFTYPE:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_IFINDEX:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_IFNAME:nlh.NLA_STRING,
    nl80211h.NL80211_ATTR_MAC:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_PREV_BSSID:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_KEY:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_KEY_DATA:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_KEY_IDX:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_KEY_CIPHER:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_KEY_DEFAULT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_KEY_SEQ:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_KEY_TYPE:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_BEACON_INTERVAL:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_DTIM_PERIOD:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_BEACON_HEAD:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_BEACON_TAIL:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_STA_AID:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_STA_FLAGS:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_STA_LISTEN_INTERVAL:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_STA_SUPPORTED_RATES:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_STA_PLINK_ACTION:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_STA_VLAN:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_MNTR_FLAGS:nlh.NLA_U32, # it seems to work by adding this attribute for each flag
    nl80211h.NL80211_ATTR_MESH_ID:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_MPATH_NEXT_HOP:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_REG_ALPHA2:nlh.NLA_STRING,
    nl80211h.NL80211_ATTR_REG_RULES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_BSS_CTS_PROT:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_BSS_SHORT_PREAMBLE:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_BSS_SHORT_SLOT_TIME:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_BSS_BASIC_RATES:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_BSS_HT_OPMODE:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_MESH_CONFIG:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_SUPPORT_MESH_AUTH:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_HT_CAPABILITY:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_MGMT_SUBTYPE:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_IE:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_SCAN_FREQUENCIES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_SCAN_SSIDS:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_SSID:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_AUTH_TYPE:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_REASON_CODE:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_FREQ_FIXED:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_TIMED_OUT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_USE_MFP:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_STA_FLAGS2:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_CONTROL_PORT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_CONTROL_PORT_ETHERTYPE:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_CONTROL_PORT_NO_ENCRYPT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_PRIVACY:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_CIPHER_SUITE_GROUP:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WPA_VERSIONS:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_PID:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_4ADDR:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_PMKID:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_DURATION:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_COOKIE:nlh.NLA_U64,
    nl80211h.NL80211_ATTR_TX_RATES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_FRAME:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_FRAME_MATCH:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_PS_STATE:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_CQM:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_LOCAL_STATE_CHANGE:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_AP_ISOLATE:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_WIPHY_TX_POWER_SETTING:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_TX_POWER_LEVEL:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_FRAME_TYPE:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_WIPHY_ANTENNA_TX:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_ANTENNA_RX:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_MCAST_RATE:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_OFFCHANNEL_TX_OK:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_KEY_DEFAULT_TYPES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_WIPHY_ANTENNA_AVAIL_TX:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WIPHY_ANTENNA_AVAIL_RX:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_WOWLAN_TRIGGERS:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_STA_PLINK_STATE:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_SCHED_SCAN_INTERVAL:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_REKEY_DATA:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_SCAN_SUPP_RATES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_HIDDEN_SSID:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_IE_PROBE_RESP:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_IE_ASSOC_RESP:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_ROAM_SUPPORT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_SCHED_SCAN_MATCH:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_TX_NO_CCK_RATE:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_TDLS_ACTION:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_TDLS_DIALOG_TOKEN:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_TDLS_OPERATION:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_TDLS_SUPPORT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_TDLS_EXTERNAL_SETUP:nlh.NLA_FLAG,
    #nl80211h.NL80211_ATTR_TDLS_INITIATOR:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_DONT_WAIT_FOR_ACK:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_PROBE_RESP:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_DFS_REGION:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_DISABLE_HT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_HT_CAPABILITY_MASK:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_NOACK_MAP:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_INACTIVITY_TIMEOUT:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_BG_SCAN_PERIOD:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_WDEV:nlh.NLA_U64,
    nl80211h.NL80211_ATTR_USER_REG_HINT_TYPE:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_SAE_DATA:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_VHT_CAPABILITY:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_SCAN_FLAGS:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_P2P_CTWINDOW:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_P2P_OPPPS:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_ACL_POLICY:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_MAC_ADDRS:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_STA_CAPABILITY:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_STA_EXT_CAPABILITY:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_SPLIT_WIPHY_DUMP:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_DISABLE_VHT:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_VHT_CAPABILITY_MASK:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_MDID:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_IE_RIC:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_PEER_AID:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_CH_SWITCH_COUNT:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_CH_SWITCH_BLOCK_TX:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_CSA_IES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_CSA_C_OFF_BEACON:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_CSA_C_OFF_PRESP:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_STA_SUPPORTED_CHANNELS:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_STA_SUPPORTED_OPER_CLASSES:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_HANDLE_DFS:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_SUPPORTED_IFTYPES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_SOFTWARE_IFTYPES:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_WIPHY_BANDS:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_SUPPORTED_COMMANDS:nlh.NLA_NESTED,
    nl80211h.NL80211_ATTR_MAX_NUM_SCAN_SSIDS:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_GENERATION:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_BSS:nlh.NLA_NESTED,
    #nl80211h.NL80211_ATTR_MAC:nlh.NLA_STRING, # we don't nl want to parse this
    nl80211h.NL80211_ATTR_CIPHER_SUITES:nlh.NLA_SET_U32, # my own - set of U32s
    nl80211h.NL80211_ATTR_STA_INFO:nlh.NLA_NESTED,
    #### defined in kernel v4 nl80211_h
    nl80211h.NL80211_ATTR_OPMODE_NOTIF:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_VENDOR_ID:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_VENDOR_SUBCMD:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_VENDOR_DATA:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_QOS_MAP:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_MAC_HINT:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_WIPHY_FREQ_HINT:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_TDLS_PEER_CAPABILITY:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_SOCKET_OWNER:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_CSA_C_OFFSETS_TX:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_USE_RRM:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_TSID:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_USER_PRIO:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_ADMITTED_TIME:nlh.NLA_U16,
    nl80211h.NL80211_ATTR_SMPS_MODE:nlh.NLA_U8,
    nl80211h.NL80211_ATTR_MAC_MASK:nlh.NLA_UNSPEC,
    nl80211h.NL80211_ATTR_WIPHY_SELF_MANAGED_REG:nlh.NLA_FLAG,
    nl80211h.NL80211_ATTR_NETNS_FD:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_SCHED_SCAN_DELAY:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_REG_INDOOR:nlh.NLA_FLAG,
    #nl80211h.NL80211_ATTR_PBSS:nlh.NLA_FLAG
}

# policy for sta_info
# static struct nla_policy stats_policy[NL80211_STA_INFO_MAX + 1] = {
nl80211_sta_info_polcy = {
    nl80211h.NL80211_STA_INFO_INACTIVE_TIME:nlh.NLA_U32,
    nl80211h.NL80211_STA_INFO_RX_BYTES:nlh.NLA_U32,
    nl80211h.NL80211_STA_INFO_TX_BYTES:nlh.NLA_U32,
    nl80211h.NL80211_STA_INFO_RX_PACKETS:nlh.NLA_U32,
    nl80211h.NL80211_STA_INFO_TX_PACKETS:nlh.NLA_U32,
    nl80211h.NL80211_STA_INFO_SIGNAL:nlh.NLA_U8,
    nl80211h.NL80211_STA_INFO_TX_BITRATE:nlh.NLA_NESTED,
    nl80211h.NL80211_STA_INFO_LLID:nlh.NLA_U16,
    nl80211h.NL80211_STA_INFO_PLID:nlh.NLA_U16,
    nl80211h.NL80211_STA_INFO_PLINK_STATE:nlh.NLA_U8
}

#static struct nla_policy rate_policy[NL80211_RATE_INFO_MAX + 1] = {
#[NL80211_RATE_INFO_BITRATE]      = { .type = NLA_U16  },
#[NL80211_RATE_INFO_MCS]          = { .type = NLA_U8   },
#[NL80211_RATE_INFO_40_MHZ_WIDTH] = { .type = NLA_FLAG },
#[NL80211_RATE_INFO_SHORT_GI]     = { .type = NLA_FLAG },
#};

# policy for the key attributes
#static const struct nla_policy nl80211_key_policy[NL80211_KEY_MAX + 1] = {
nl80211_key_policy = {
    nl80211h.NL80211_KEY_DATA:nlh.NLA_UNSPEC,
    nl80211h.NL80211_KEY_IDX:nlh.NLA_U8,
    nl80211h.NL80211_KEY_CIPHER:nlh.NLA_U32,
    nl80211h.NL80211_KEY_SEQ:nlh.NLA_UNSPEC,
    nl80211h.NL80211_KEY_DEFAULT:nlh.NLA_FLAG,
    nl80211h.NL80211_KEY_DEFAULT_MGMT:nlh.NLA_FLAG,
    nl80211h.NL80211_KEY_TYPE:nlh.NLA_U32,
    nl80211h.NL80211_KEY_DEFAULT_TYPES:nlh.NLA_NESTED
}

# policy for the key default flags
#static const struct nla_policy
nl80211_key_default_policy = {
    nl80211h.NL80211_KEY_DEFAULT_TYPE_UNICAST:nlh.NLA_FLAG,
    nl80211h.NL80211_KEY_DEFAULT_TYPE_MULTICAST:nlh.NLA_FLAG
}

#/* policy for WoWLAN attributes */
#static const struct nla_policy nl80211_wowlan_policy[NUM_NL80211_WOWLAN_TRIG] = {
nl80211_wowlan_trig_policy = {
    nl80211h.NL80211_WOWLAN_TRIG_ANY:nlh.NLA_FLAG,
    nl80211h.NL80211_WOWLAN_TRIG_DISCONNECT:nlh.NLA_FLAG,
    nl80211h.NL80211_WOWLAN_TRIG_MAGIC_PKT:nlh.NLA_FLAG,
    nl80211h.NL80211_WOWLAN_TRIG_PKT_PATTERN:nlh.NLA_NESTED,
    nl80211h.NL80211_WOWLAN_TRIG_GTK_REKEY_FAILURE:nlh.NLA_FLAG,
    nl80211h.NL80211_WOWLAN_TRIG_EAP_IDENT_REQUEST:nlh.NLA_FLAG,
    nl80211h.NL80211_WOWLAN_TRIG_4WAY_HANDSHAKE:nlh.NLA_FLAG,
    nl80211h.NL80211_WOWLAN_TRIG_RFKILL_RELEASE:nlh.NLA_FLAG,
    nl80211h.NL80211_WOWLAN_TRIG_TCP_CONNECTION:nlh.NLA_NESTED
    #nl80211h.NL80211_WOWLAN_TRIG_NET_DETECT:nlh.NLA_NESTED
}

#static const struct nla_policy nl80211_wowlan_tcp_policy[NUM_NL80211_WOWLAN_TCP] = {
nl80211_wowlan_tcp_policy = {
    nl80211h.NL80211_WOWLAN_TCP_SRC_IPV4:nlh.NLA_U32,
    nl80211h.NL80211_WOWLAN_TCP_DST_IPV4:nlh.NLA_U32,
    nl80211h.NL80211_WOWLAN_TCP_DST_MAC:nlh.NLA_UNSPEC,
    nl80211h.NL80211_WOWLAN_TCP_SRC_PORT:nlh.NLA_U16,
    nl80211h.NL80211_WOWLAN_TCP_DST_PORT:nlh.NLA_U16,
    nl80211h.NL80211_WOWLAN_TCP_DATA_PAYLOAD:nlh.NLA_UNSPEC,
    nl80211h.NL80211_WOWLAN_TCP_DATA_PAYLOAD_SEQ:nlh.NLA_UNSPEC,
    nl80211h.NL80211_WOWLAN_TCP_DATA_PAYLOAD_TOKEN:nlh.NLA_UNSPEC,
    nl80211h.NL80211_WOWLAN_TCP_DATA_INTERVAL:nlh.NLA_U32,
    nl80211h.NL80211_WOWLAN_TCP_WAKE_PAYLOAD:nlh.NLA_UNSPEC,
    nl80211h.NL80211_WOWLAN_TCP_WAKE_MASK:nlh.NLA_UNSPEC
}

#/* policy for coalesce rule attributes */
#static const struct nla_policy nl80211_coalesce_policy[NUM_NL80211_ATTR_COALESCE_RULE] = {
nl80211_coalesce_policy={
    nl80211h.NL80211_ATTR_COALESCE_RULE_DELAY:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_COALESCE_RULE_CONDITION:nlh.NLA_U32,
    nl80211h.NL80211_ATTR_COALESCE_RULE_PKT_PATTERN:nlh.NLA_NESTED
}

#/* policy for GTK rekey offload attributes */
#static const struct nla_policy nl80211_rekey_policy[NUM_NL80211_REKEY_DATA] = {
nl80211_rekey_policy = {
    nl80211h.NL80211_REKEY_DATA_KEK:nlh.NLA_UNSPEC,
    nl80211h.NL80211_REKEY_DATA_KCK:nlh.NLA_UNSPEC,
    nl80211h.NL80211_REKEY_DATA_REPLAY_CTR:nlh.NLA_UNSPEC
}

#static const struct nla_policy nl80211_match_policy[NL80211_SCHED_SCAN_MATCH_ATTR_MAX + 1] = {
nl80211_sched_scan_match_policy = {
    nl80211h.NL80211_SCHED_SCAN_MATCH_ATTR_SSID:nlh.NLA_UNSPEC,
    nl80211h.NL80211_SCHED_SCAN_MATCH_ATTR_RSSI:nlh.NLA_U32
}

#static const struct nla_policy nl80211_plan_policy[NL80211_SCHED_SCAN_PLAN_MAX + 1] = {
nl80211_sched_scan_plan_policy = {
    nl80211h.NL80211_SCHED_SCAN_PLAN_INTERVAL:nlh.NLA_U32,
    nl80211h.NL80211_SCHED_SCAN_PLAN_ITERATIONS:nlh.NLA_U32
}



#!/usr/bin/env python
""" info.py
Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without
modifications, are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of
    any contributors may be used to endorse or promote products derived from
    this software without specific prior written permission.

Example for displaying device details

"""

from __future__ import print_function  # python 2to3 compability
import argparse as ap                  # cli arg parsing
import sys                             # cli exiting
import pyric                           # pyric error (and ecode EUNDEF)
import pyric.pyw as pyw                # for iw functionality
from pyric.utils.channels import rf2ch # rf to channel conversion

def execute(dev,itype):
    # ensure dev is a wireless interfaces
    wifaces = pyw.winterfaces()
    if dev not in wifaces:
        print("Device {0} is not wireless, use one of {1}".format(dev,wifaces))

    # get info dicts
    dinfo = pyw.devinfo(dev)
    card = dinfo['card']
    pinfo = pyw.phyinfo(card)
    iinfo = pyw.ifinfo(card)

    if itype == 'all' or itype == 'if':
        msg = "Interface {0}\n".format(card.idx)
        msg += "\tDriver: {0} Chipset: {1}\n".format(iinfo['driver'],iinfo['chipset'])
        msg += "\tHW Addr: {0} Manufacturer: {1}\n".format(iinfo['hwaddr'],
                                                           iinfo['manufacturer'])
        msg += "\tInet: {0} Bcast: {1} Mask: {2}\n".format(iinfo['inet'],
                                                           iinfo['bcast'],
                                                           iinfo['mask'])
        print(msg)

    if itype == 'all' or itype == 'dev':
        msg = "Device {0}\n".format(card.dev)
        msg += "\tifindex: {0}\n".format(card.idx)
        msg += "\twdev: {0}\n".format(dinfo['wdev'])
        msg += "\taddr: {0}\n".format(dinfo['mac'])
        msg += "\tmode: {0}\n".format(dinfo['mode'])
        msg += "\twiphy: {0}\n".format(card.phy)
        if dinfo['mode'] != 'managed': msg += "\tDevice not associated\n"
        else:
            msg += "\tchannel: {0} ({1} MHz), width: {2}, CF: {3} MHz\n".format(rf2ch(dinfo['RF']),
                                                                                dinfo['RF'],
                                                                                dinfo['CHW'],
                                                                                dinfo['CF'])
        print(msg)

    if itype == 'all' or itype == 'phy':
        msg = "Wiphy phy{0}\n".format(card.phy)
        msg += "\tGeneration: {0}m Coverage Class: {1}\n".format(pinfo['generation'],
                                                                 pinfo['cov_class'])
        msg += "\tMax # scan SSIDs: {0}\n".format(pinfo['scan_ssids'])
        msg += "\tRetry Short: {0}, Long: {1}\n".format(pinfo['retry_short'],
                                                        pinfo['retry_long'])
        msg += "\tThreshold Frag: {0}, RTS: {1}\n".format(pinfo['frag_thresh'],
                                                          pinfo['rts_thresh'])
        msg += "\tSupported Modes:\n"
        for mode in pinfo['modes']: msg += "\t  * {0}\n".format(mode)
        msg += "\tSupported Commands:\n"
        for cmd in pinfo['commands']: msg += "\t  * {0}\n".format(cmd)
        msg += "\tSupported Ciphers:\n"
        for cipher in pinfo['ciphers']: msg += "\t  * {0}\n".format(cipher)
        for band in pinfo['bands']:
            msg += "\tBand {0}: (HT: {1} VHT: {2})\n".format(band,
                                                             pinfo['bands'][band]['HT'],
                                                             pinfo['bands'][band]['VHT'])
            msg += "\t   Rates:\n"
            for rate in pinfo['bands'][band]['rates']:
                msg += "\t    * {0} Mbps\n".format(rate)
            msg += "\t   Frequencies:\n"
            for i,rf in enumerate(pinfo['bands'][band]['rfs']):
                dbm = pinfo['bands'][band]['rf-data'][i]['max-tx']
                msg += "\t    * {0} MHz ({1} dBm)".format(rf,dbm)
                if not pinfo['bands'][band]['rf-data'][i]['enabled']:
                    msg += " (disabled)\n"
                else:
                    msg += "\n"
        print(msg)

if __name__ == '__main__':
    # create arg parser and parse command line args
    print("Wireless Device Info Display using PyRIC v{0}".format(pyric.version))
    argp = ap.ArgumentParser(description="Wireless Device Data")
    argp.add_argument('-d','--dev',help="Wireless Device")
    argp.add_argument('-t','--type',help="Info type one of {all|if|dev|phy}")
    args = argp.parse_args()
    usage = "usage: python info.py -d <dev> [-t one of {all|if|dev|phy}]"
    try:
        dname = args.dev
        infotype = args.type
        if dname is None:
            print(usage)
            sys.exit(0)
        if infotype is None: infotype = 'all'
        if infotype not in ['all','if','dev','phy']:
            print(usage)
            sys.exit(0)
        execute(dname,infotype)
    except pyric.error as e:
        print(e)


#!/usr/bin/env python
""" nlhelp.py: nl80211 help functions

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

A set of functions to assist in finding info on nl80211 commands and attributes.
These are stored in the "data" files commands.help and attributes.help which are
json files.

"""

__name__ = 'nlhelp'
__license__ = 'GPLv3'
__version__ = '0.0.1'
__date__ = 'August 2014'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import os
import json
import pyric
import pyric.net.wireless.nl80211_h as nl80211h

# get current working directory
fpath = os.path.dirname(os.path.realpath(__file__))

# read in the files here
cmdpath = os.path.join(fpath,'commands.help')
commands = None   # cmd -> desc,attributes used dict
cmdlookup = None  # reverse lookup for command constants
cin = None
try:
    # first three lines are comments, 4th line is empty
    cin = open(cmdpath,'r')
    for _ in range(4): _in = cin.readline()
    commands = json.loads(cin.readline())
    cmdlookup = json.loads(cin.readline())
except:
    raise pyric.error(pyric.EUNDEF,"Failed to process commands.help")
finally:
    if cin: cin.close()

attrpath = os.path.join(fpath,'attributes.help')
attributes = None # attr -> desc, commands used by, datatype
attrlookup = None # reverse lookup for attribute constants
ain = None
try:
    # first three lines are comments, 3th line is empty
    ain = open(attrpath,'r')
    for _ in range(4): _in = ain.readline()
    attributes = json.loads(ain.readline())
    attrlookup = json.loads(ain.readline())
except:
    raise pyric.error(pyric.EUNDEF, "Failed to process attributes.help")
finally:
    if ain: ain.close()

def command(cmd):
    """
      shows help on command can either be the full name i.e. NL80211_CMD_GET_WIPHY
      or a shortened version GET_WIPHY
     :param cmd: command to show description of.
     :returns: description of command, attributes used in command and the constant
     that refers to the command
    """
    try:
        cmd = cmd.upper().replace('@','') # in the event it comes from cmdbynum
        if not cmd.startswith("NL80211_CMD_"): cmd = "@NL80211_CMD_" + cmd
        else: cmd = '@' + cmd
        entry = commands[cmd]
        attrs = ", ".join([attr.replace('%','') for attr in entry['attrs']])
        out = "{0}\tValue={1}\n".format(cmd,eval('nl80211h.' + cmd[1:]))
        out += "------------------------------------------------------\n"
        out += "Description: {0}\n".format(entry['desc'])
        out += "------------------------------------------------------\n"
        out += "Attributes: {0}".format(attrs)
        return out
    except KeyError:
        return "No entry found for command {0}".format(cmd)
    except AttributeError:
        return "{0} not found in nl80211_h".format(cmd)

def cmdbynum(n):
    """
     reverse lookup n to corresponding command variable
     :param n: integer value to search for
     :returns: string representation of the command variable corresponding to n
    """
    return cmdlookup[str(n)]

def attribute(attr):
    """
     shows help on attribute can either be the full name i.e. NL80211_ATTR_MAC
     or a shortened version MAC
     :param attr: attribute to show description of.
     :returns: description of attribute, commands that use the attribute, datatype
      of the attribute and the constant that refers to the attribute
      """
    try:
        attr = attr.upper().replace('@','')  # in the event it comes from attrbynum
        if not attr.startswith("NL80211_ATTR_"): attr = "@NL80211_ATTR_" + attr
        else: attr = '@' + attr
        entry = attributes[attr]
        cmds = ", ".join([cmd.replace('%', '') for cmd in entry['cmds']])
        out = "{0}\tValue={1}\tDatatype={2}\n".format(attr,
                                                      eval('nl80211h.' + attr[1:]),
                                                      entry['type'])
        out += "------------------------------------------------------\n"
        out += "Description: {0}\n".format(entry['desc'])
        out += "------------------------------------------------------\n"
        out += "Commands: {0}".format(cmds)
        return out
    except KeyError:
        return "No entry found for attribute {0}".format(attr)
    except AttributeError:
        return "{0} not found in nl80211_h".format(attr)

def attrbynum(n):
    """
     reverse lookup n to corresponding attribute variable
     :param n: integer value to search for
     :returns: string representation of the command variable corresponding to n
    """
    return attrlookup[str(n)][0]

def search(tkn):
    """
     searches for and returns any commands,attributes with tkn
     :param tkn:
     :returns: a list of commands,attributes with tkn in them
    """
    tkn = tkn.upper()
    if len(tkn)  < 3:
        raise pyric.error(pyric.EUNDEF,"{0} is to ambiguous".format(tkn))
    found = [cmd for cmd in commands if tkn in cmd]
    found += [attr for attr in attributes if tkn in attr]
    return found


#!/usr/bin/env python
""" net
Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

 wireless 0.0.1
  desc: linux port of nl80211.h, nl80211.c
 includes: rfkill_h 0.0.1 nl80211_h 0.0.4 nl80211_c 0.0.2 wlan 0.0.2
 changes:
  o added nl80211_c to handle attribute policies
   - added nl80211_parse_freqs to parse out supported frequencies

"""

__name__ = 'wireless'
__license__ = 'GPLv3'
__version__ = '0.0.4'
__date__ = 'June 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'


#!/usr/bin/env python
""" libio provides ioctl socket & send/recv functionality

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

Basic wrappers providing functionality for socket creation/deletion and transfer
i.e. send/recv w.r.t ioctl calls

"""

__name__ = 'libio'
__license__ = 'GPLv3'
__version__ = '0.0.1'
__date__ = 'April 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import socket
import struct
import errno
from fcntl import ioctl

class error(EnvironmentError): pass

def io_socket_alloc():
    """
     create a socket for ioctl calls
     :returns: an io socket
    """
    return socket.socket(socket.AF_INET,socket.SOCK_DGRAM)

def io_socket_free(iosock):
    """ close the socket """
    if iosock: iosock.close()
    return None

def io_transfer(iosock,flag,ifreq):
    """
     send & recieve an ifreq struct
     :param iosock: io socket
     :param flag: sockios control call
     :param ifreq: ifreq to send
     :returns: an the ifreq struct recieved
    """
    try:
        return ioctl(iosock.fileno(),flag,ifreq)
    except (AttributeError,struct.error) as e:
        # either sock is not valid or a bad value passed to ifreq
        if e.message.find('fileno'): raise error(errno.ENOTSOCK,"Bad socket")
        else: raise error(errno.EINVAL,e)
    except IOError as e:
        # generally device cannot be found sort but can also be
        # permissions etc, catch and reraise as our own
        if e.errno is not None: # just in case we have a none 2-tuple error
            raise error(e.errno,e.strerror)
        else:
            raise error(-1,e)
    except Exception as e:
        # blanket catchall
        raise error(-1,e.args[0])



#!/usr/bin/env python
""" attributes defines netlink attribute policies and functions.

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

For lack of a better place to put these, this defines attribute datatypes from
genetlink.h and imports those defined in nl80211_c.

NOTE: I only use the datatype ignoring minlength, maxlength

"""

__name__ = 'attributes'
__license__ = 'GPLv3'
__version__ = '0.0.2'
__date__ = 'April 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import pyric.net.netlink_h as nlh
import pyric.net.genetlink_h as genlh
import pyric.net.wireless.nl80211_c as nl80211c

def nla_datatype(policy,attr):
    """
     determines the appropriate attribute datatype as found in policy
     :param policy: policy name
     :param attr: attribute type
     :returns: a datatype as specified in netlink_h
     NOTE: will return NLA_UNSPEC if given attr can not be found in policy
    """
    try:
        return nla_dts[policy][attr]
    except (KeyError,IndexError):
        return nlh.NLA_UNSPEC

# map string names to datatype lists
nla_dts = {}
nla_dts_set = {}

#### CTRL_ATTR_*
# commented out below to determine if nested _OPS and _MCAST_GROUPS
# was causing an infinite loop in nla_parse_nested
nla_dts['ctrl_attr'] = {genlh.CTRL_ATTR_UNSPEC:nlh.NLA_UNSPEC,
                        genlh.CTRL_ATTR_FAMILY_ID:nlh.NLA_U16,
                        genlh.CTRL_ATTR_FAMILY_NAME:nlh.NLA_STRING,
                        genlh.CTRL_ATTR_VERSION:nlh.NLA_U32,
                        genlh.CTRL_ATTR_HDRSIZE:nlh.NLA_U32,
                        genlh.CTRL_ATTR_MAXATTR:nlh.NLA_U32,
                        genlh.CTRL_ATTR_OPS:nlh.NLA_NESTED,
                        #genlh.CTRL_ATTR_OPS: nlh.NLA_UNSPEC,
                        genlh.CTRL_ATTR_MCAST_GROUPS:nlh.NLA_NESTED}
                        #genlh.CTRL_ATTR_MCAST_GROUPS: nlh.NLA_UNSPEC}

#### CTRL_ATTR_OP_*
nla_dts['ctrl_attr_op'] = {genlh.CTRL_ATTR_OP_UNSPEC:nlh.NLA_UNSPEC,
                           genlh.CTRL_ATTR_OP_ID:nlh.NLA_U32,
                           genlh.CTRL_ATTR_OP_FLAGS:nlh.NLA_U32}

#### CTRL_ATTR_MCAST_*
nla_dts['ctrl_attr_mcast'] = {genlh.CTRL_ATTR_MCAST_GRP_UNSPEC:nlh.NLA_UNSPEC,
                              genlh.CTRL_ATTR_MCAST_GRP_NAME:nlh.NLA_STRING,
                              genlh.CTRL_ATTR_MCAST_GRP_ID:nlh.NLA_U32}

nla_dts['nl80211_attr'] = nl80211c.nl80211_policy


#!/usr/bin/env python

""" genetlink_h.py: port of netlink.h public header
/*
 * NETLINK      Generic Netlink Family
 *
 *              Authors:        Jamal Hadi Salim
 *                              Thomas Graf <tgraf@suug.ch>
 *                              Johannes Berg <johannes@sipsolutions.net>
 */

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

A port of genetlink.h to python and defines the nla_policy for generic netlink
attributes.

"""

__name__ = 'genetlink_h'
__license__ = 'GPLv3'
__version__ = '0.0.1'
__date__ = 'March 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import struct

GENL_NAMSIZ	= 16 # length of family name
GENL_MIN_ID	= 0x10 # hardcoded from netlink_h
GENL_MAX_ID	= 1023

"""
struct genlmsghdr {
	__u8	cmd;
	__u8	version;
	__u16	reserved;
};
"""
genl_genlmsghdr = "BBH"
GENLMSGHDRLEN = struct.calcsize(genl_genlmsghdr)
def genlmsghdr(cmd,vers=1):
    """
     create a generic netlink header
     :param cmd: message type of genetlink service
     :param vers: revision value for backward compatability
     :returns: packed generic netlink header
    """
    return struct.pack(genl_genlmsghdr,cmd,vers,0)

#GENL_HDRLEN	NLMSG_ALIGN(sizeof(struct genlmsghdr))

GENL_ADMIN_PERM		= 0x01
GENL_CMD_CAP_DO		= 0x02
GENL_CMD_CAP_DUMP	= 0x04
GENL_CMD_CAP_HASPOL	= 0x08

# List of reserved static generic netlink identifiers:
GENL_ID_GENERATE  = 0
GENL_ID_CTRL	  = 0x10 # hardcoded from netlink_h
GENL_ID_VFS_DQUOT = GENL_ID_CTRL + 1
GENL_ID_PMCRAID	  = GENL_ID_CTRL + 2

#Controller
CTRL_CMD_UNSPEC       =  0
CTRL_CMD_NEWFAMILY    =  1
CTRL_CMD_DELFAMILY    =  2
CTRL_CMD_GETFAMILY    =  3
CTRL_CMD_NEWOPS       =  4
CTRL_CMD_DELOPS       =  5
CTRL_CMD_GETOPS       =  6
CTRL_CMD_NEWMCAST_GR  =  7
CTRL_CMD_DELMCAST_GRP =  8
CTRL_CMD_GETMCAST_GRP =  9 # unused
__CTRL_CMD_MAX        = 10
CTRL_CMD_MAX          = __CTRL_CMD_MAX - 1

CTRL_ATTR_UNSPEC       = 0
CTRL_ATTR_FAMILY_ID    = 1
CTRL_ATTR_FAMILY_NAME  = 2
CTRL_ATTR_VERSION      = 3
CTRL_ATTR_HDRSIZE      = 4
CTRL_ATTR_MAXATTR      = 5
CTRL_ATTR_OPS          = 6
CTRL_ATTR_MCAST_GROUPS = 7
__CTRL_ATTR_MAX        = 9
CTRL_ATTR_MAX          = __CTRL_ATTR_MAX - 1

CTRL_ATTR_OP_UNSPEC = 0
CTRL_ATTR_OP_ID     = 1
CTRL_ATTR_OP_FLAGS  = 2
__CTRL_ATTR_OP_MAX  = 3
CTRL_ATTR_OP_MAX    = __CTRL_ATTR_OP_MAX - 1
nla_policy_attr_op = []

CTRL_ATTR_MCAST_GRP_UNSPEC = 0
CTRL_ATTR_MCAST_GRP_NAME   = 1
CTRL_ATTR_MCAST_GRP_ID     = 2
__CTRL_ATTR_MCAST_GRP_MAX  = 3
CTRL_ATTR_MCAST_GRP_MAX    = __CTRL_ATTR_MCAST_GRP_MAX - 1
nla_policy_attr_mcast = []


#!/usr/bin/env python
""" lib lib subpackage

Copyright (C) 2016  Dale V. Patterson (wraith.wireless@yandex.com)

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

Redistribution and use in source and binary forms, with or without modifications,
are permitted provided that the following conditions are met:
 o Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 o Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 o Neither the name of the orginal author Dale V. Patterson nor the names of any
   contributors may be used to endorse or promote products derived from this
   software without specific prior written permission.

lib subpackage

 lib 0.0.2
  desc: lib subpackage
 includes: libnl 0.1.1 libio 0.0.1
 changes:
  o added libio
  o updated libnl
   - added nlmsg_fromstream
   - added NLSocket class
   - added partial support of nested attributes
  o added functionality to modify rx,tx and timeout after socket creation
  o update libnl.py
   - remove nla_* from GENLMsg stand-alone functions as this was my original
    intent where the classes should only be 'placeholders', similar to C structs
    and not full blow objects
   - added some error checking to nla_parse_nested, raising and handling error
    in situations where attribute len is 0
   - fixed nlmsg_fromstream to handle non-ack message parsing with a length of 36
  o libnl and libnl now report their own errors rather than use pyric.error
  o added support for NLA_SET
"""

__name__ = 'lib'
__license__ = 'GPLv3'
__version__ = '0.0.2'
__date__ = 'April 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'


# PyRIC v 1.6 root Distribution directory
# Do not import from this directory i.e from PyRIC import foobar
# To install
#   use pip ('sudo pip install PyRIC')
# or download latest tarbal and untar.
# excute either sudo python setup.py or rund from the PyRIC/pyric directory



#!/usr/bin/env python
""" pyw.unittest.py: unittest

Define unittest functions for pyw

Assumptions:
 o user has root privileges
 o user has set up global variables to system
 o private functions are tested via methods
 o persistent sockets are tested via one-time methods that is, one-time socket
   calls result in netlink socket creation, usage and deletion

usage:
 sudo python pyw.unittest.py -v

Results as of 24-July-16

sudo python pyw.unittest.py
Testing PyRIC v0.1.5 pyw v0.1.8 on Python 2.7.12
...................................................................................
----------------------------------------------------------------------
Ran 83 tests in 5.919s

OK

NOTE:
 1) functions disconnect and link require a connection, they are tested/confirmed
    manually
 2) function devadd (and subsequently devset) are commented out. There is a
   peculiar behavior in netlink/nl80211 (appearing in kernel 4.4.0-x) where
   regardless of the name passed to create a new device as in
    iw phy <phy> interface add <new dev> type <new mode>
    pyw.devadd(<card>, <new name>, <new mode>)
   the kernel or driver or whoever will instead assign a predicatable name
   of the form:
    wlx00c0ca59afa7
   devadd has been fixed but this is currently not reflected in the below unittests
   it has been manually tested and confirmed
"""
from __future__ import print_function  # python 2to3 compability

#__name__ = 'pyw.unittest'
__license__ = 'GPLv3'
__version__ = '0.0.3'
__date__ = 'July 2016'
__author__ = 'Dale Patterson'
__maintainer__ = 'Dale Patterson'
__email__ = 'wraith.wireless@yandex.com'
__status__ = 'Production'

import unittest
import time
import pyric
from pyric import error
import pyric.pyw as pyw
import pyric.utils.channels as channels
import pyric.net.wireless.wlan as wlan
import sys

# modify below to fit your system
pri = {'dev':'alfa0',
       'mac':'00:c0:ca:59:af:a6',
       'ifindex':18,
       'phy':7,
       'driver':'rt2800usb',
       'chipset':'Ralink RT2870/3070',
       'mode':'managed',
       'tx':20,
       'freqs':[2412,2417,2422,2427,2432,2437,2442,2447,2452,2457,2462,2467,2472,2484],
       #'freqs':[2412,2417,2422,2427,2432,2437,2442,2447,2452,2457,2462,2467,2472,
       #         5180,5200,5220,5240,5260,5280,5300,5320, 5500,5520,5540,5560,
       #         5580,5600,5620,5640,5660,5680,5700,5745,5765,5785,5805, 5825],
       'stds':['b','g','n'],
       'modes':['ibss', 'managed', 'AP', 'AP VLAN', 'wds','monitor', 'mesh'],
       'ip':'10.0.0.2',
       'bcast':'10.0.0.255',
       'mask':'255.255.255.0'}
newhw = '00:c0:ca:60:b0:a7'
newip = '10.0.0.3'
nics = ['eth0','lo','wlan0','alfa0']
enics = ['eth0','lo']
wnics = ['wlan0','alfa0']
inics = ['foo0','bar0']
regdom = '00'
newregdom = 'BO'

# test functions interfaces and isinterface
class InterfaceTestCase(unittest.TestCase):
    def test_enuminterfaces(self):
        for nic in nics: self.assertTrue(nic in pyw.interfaces())
    def test_isinterface(self):
        for nic in pyw.interfaces(): self.assertTrue(pyw.isinterface(nic))
    def test_not_isinterface(self):
        for inic in inics: self.assertFalse(pyw.isinterface(inic))
    def test_ininterfaces(self):
        for nic in nics: self.assertIn(nic,pyw.interfaces())

# test functions winterfaces and iswireless
class WInterfaceTestCase(unittest.TestCase):
    def test_enumwinterfaces(self):
        for wnic in wnics: self.assertTrue(wnic in pyw.winterfaces())
    def test_iswinterface(self):
        for wnic in pyw.winterfaces(): self.assertTrue(pyw.iswireless(wnic))
    def test_not_iswinterface(self):
        for nic in inics + enics: self.assertFalse(pyw.iswireless(nic))
    def test_inwinterfaces(self):
        for wnic in pyw.winterfaces(): self.assertIn(wnic,pyw.winterfaces())

# test regget, regset
class RegDomTestCase(unittest.TestCase):
    def test_regget(self): self.assertEqual(regdom,pyw.regget())
    def test_regset(self):
        self.assertEqual(None,pyw.regset(newregdom))
        time.sleep(0.25) # give sleep time
        self.assertEqual(newregdom,pyw.regget())
        self.assertEqual(None,pyw.regset(regdom))
        time.sleep(0.25) # give sleep time
        self.assertEqual(regdom, pyw.regget())

# test getcard,validcard
class GetCardTestCase(unittest.TestCase):
    def test_getcard(self):
        for wnic in wnics: self.assertIsNotNone(pyw.getcard(wnic))
    def test_notacard(self):
        for enic in enics: self.assertRaises(error,pyw.getcard,enic)
    def test_validcard(self):
        for wnic in wnics: self.assertTrue(pyw.validcard(pyw.getcard(wnic)))

# super class for test cases requiring a Card object
class CardTestCase(unittest.TestCase):
    def setUp(self): self.card = pyw.getcard(pri['dev'])
    def tearDown(self): pass


# test macget
class MacGetTestCase(CardTestCase):
    def test_macget(self):
        self.assertEquals(pri['mac'],pyw.macget(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.macget,'bad0')

# test macset
class MacSetTestCase(CardTestCase):
    def setUp(self):
        CardTestCase.setUp(self)
        pyw.down(self.card)
    def tearDown(self):
        pyw.up(self.card)
    def test_macset(self):
        self.assertEqual(newhw,pyw.macset(self.card,newhw))
        self.assertEqual(pri['mac'],pyw.macset(self.card,pri['mac']))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.macset,'bad0',newhw)
    def test_invalidmacarg(self):
        self.assertRaises(error,pyw.macset,self.card,'00:0A')

# test inetget/inetset
# testing both together as the test card is never associated thus
# never has an ip etc
# NOTE: through inetset, we get the side-effect of testing ip4set, netmaskset,
#  broadcastset
class InetGetSetTestCase(CardTestCase):
    def test_inetgetset(self):
        self.assertEquals(None,pyw.ifaddrset(self.card,pri['ip'],pri['mask'],pri['bcast']))
        self.assertEqual(pri['ip'],pyw.ifaddrget(self.card)[0])
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.ifaddrget,'bad0')
        self.assertRaises(error,pyw.ifaddrset,'bad0',pri['ip'],pri['mask'],pri['bcast'])
    def test_invalidiparg(self):
        self.assertRaises(error,pyw.ifaddrset,self.card,'192.168',pri['mask'],pri['bcast'])
    def test_invalidmaskarg(self):
        self.assertRaises(error,pyw.ifaddrset,self.card,pri['ip'],'255.255',pri['bcast'])
    def test_invalidbcastarg(self):
        self.assertRaises(error,pyw.ifaddrset,self.card,pri['ip'],pri['mask'],'192.168')

# isup, test only card check
class IsUpTestCase(CardTestCase):
    def test_invalidcardarg(self): self.assertRaises(error,pyw.isup,'bad0')

# test up/isup
class UpTestCase(CardTestCase):
    def test_up(self):
        self.assertEquals(None,pyw.up(self.card))
        self.assertTrue(pyw.isup(self.card))
    def test_invalidcardarg(self): self.assertRaises(error,pyw.up,'bad0')

# test down
class DownTestCase(CardTestCase):
    def test_down(self):
        self.assertEqual(None,pyw.down(self.card))
        self.assertFalse(pyw.isup(self.card))
    def test_invalidcardarg(self): self.assertRaises(error,pyw.down,'bad0')

# isblocked, test only valid card arg see below
class IsBlockedTestCase(unittest.TestCase):
    def test_invalidcardarg(self): self.assertRaises(error,pyw.isup,'bad0')

# test block/isblocked
class BlockTestCase(CardTestCase):
    def test_block(self):
        self.assertEquals(None,pyw.block(self.card))
        self.assertTrue(pyw.isblocked(self.card))
        self.assertEquals(None,pyw.unblock(self.card))
    def test_invalidcardarg(self): self.assertRaises(error,pyw.block,'bad0')

# test block/isblocked
class UnblockTestCase(CardTestCase):
    def test_unblock(self):
        self.assertEquals(None,pyw.unblock(self.card))
        self.assertFalse(pyw.isblocked(self.card)[0])
    def test_invalidcardarg(self): self.assertRaises(error,pyw.block,'bad0')

# test get/set power_save
class GetSetPwrSave(CardTestCase):
    def test_getsetpwrsave(self):
        pyw.pwrsaveset(self.card,True)
        self.assertTrue(pyw.pwrsaveget(self.card))
        pyw.pwrsaveset(self.card, False)
        self.assertFalse(pyw.pwrsaveget(self.card))
        pyw.pwrsaveset(self.card,True)
    def testinvalidcardarg(self):
        self.assertRaises(error,pyw.pwrsaveget,'bad0')
        self.assertRaises(error,pyw.pwrsaveset,'bad0',True)
    def testinvalidonval(self):
        self.assertRaises(error,pyw.pwrsaveset,self.card,'b')

# test covclass
# NOTE: cannot currently test set as my cards do not support it
# NOTE: covclassget uses phyinfo - if that works covclassget works

# test get/set retryshort
class RetryShortTestCase(CardTestCase):
    def test_retryshort(self):
        rs = pyw.retryshortget(self.card)
        self.assertEqual(None,pyw.retryshortset(self.card,5))
        self.assertEqual(5,pyw.retryshortget(self.card))
        self.assertEqual(None,pyw.retryshortset(self.card,rs))
        self.assertEqual(rs,pyw.retryshortget(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.retryshortget,'bad0')
        self.assertRaises(error,pyw.retryshortset,'bad0',0)
    def test_invalidlim(self):
        self.assertRaises(error,pyw.retryshortset,self.card,wlan.RETRY_MIN-1)
        self.assertRaises(error,pyw.retryshortset,self.card,wlan.RETRY_MAX+1)

# test get/set retrylong
class RetryLongTestCase(CardTestCase):
    def test_retrylong(self):
        rs = pyw.retrylongget(self.card)
        self.assertEqual(None,pyw.retrylongset(self.card,5))
        self.assertEqual(5,pyw.retrylongget(self.card))
        self.assertEqual(None,pyw.retrylongset(self.card,rs))
        self.assertEqual(rs,pyw.retrylongget(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.retrylongget,'bad0')
        self.assertRaises(error,pyw.retrylongset,'bad0',0)
    def test_invalidlim(self):
        self.assertRaises(error,pyw.retrylongset,self.card,wlan.RETRY_MIN-1)
        self.assertRaises(error,pyw.retrylongset,self.card,wlan.RETRY_MAX+1)

# test get/set RTS thresh
class RTSThreshTestCase(CardTestCase):
    def test_rtsthresh(self):
        rt = pyw.rtsthreshget(self.card)
        self.assertEqual(None,pyw.rtsthreshset(self.card,5))
        self.assertEqual(5,pyw.rtsthreshget(self.card))
        self.assertEqual(None,pyw.rtsthreshset(self.card,'off'))
        self.assertEqual('off',pyw.rtsthreshget(self.card))
        self.assertEqual(None,pyw.rtsthreshset(self.card,rt))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.rtsthreshget,'bad0')
        self.assertRaises(error,pyw.rtsthreshset,'bad0',5)
    def test_invalidthresh(self):
        self.assertRaises(error,pyw.rtsthreshset,self.card,wlan.RTS_THRESH_MIN-1)
        self.assertRaises(error,pyw.rtsthreshset,self.card,wlan.RTS_THRESH_MAX+1)
        self.assertRaises(error, pyw.rtsthreshset,self.card,'on')

# test get/set RTS thresh
class FragThreshTestCase(CardTestCase):
    def test_fragthresh(self):
        ft = pyw.fragthreshget(self.card)
        self.assertEqual(None,pyw.fragthreshset(self.card,800))
        self.assertEqual(800,pyw.fragthreshget(self.card))
        self.assertEqual(None,pyw.fragthreshset(self.card,'off'))
        self.assertEqual('off',pyw.fragthreshget(self.card))
        self.assertEqual(None,pyw.fragthreshset(self.card,ft))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.fragthreshget,'bad0')
        self.assertRaises(error,pyw.fragthreshset,'bad0',800)
    def test_invalidthresh(self):
        self.assertRaises(error,pyw.fragthreshset,self.card,wlan.FRAG_THRESH_MIN-1)
        self.assertRaises(error,pyw.fragthreshset,self.card,wlan.FRAG_THRESH_MAX+1)
        self.assertRaises(error,pyw.fragthreshset,self.card,'on')

# test get freqs
class DevFreqsTestCase(CardTestCase):
    def test_devfreqs(self):
        self.assertListEqual(pri['freqs'],pyw.devfreqs(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devfreqs,'bad0')

# test get chs
class DevCHsTestCase(CardTestCase):
    def test_devchs(self):
        [channels.rf2ch(rf) for rf in pri['freqs']]
        self.assertListEqual([channels.rf2ch(rf) for rf in pri['freqs']],
                              pyw.devchs(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devchs,'bad0')

# test get stds
class DevSTDsTestCase(CardTestCase):
    def test_devchs(self):
        self.assertListEqual(pri['stds'],pyw.devstds(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devstds,'bad0')

# test get modes
class DevModesTestCase(CardTestCase):
    def test_devmodes(self):
        self.assertListEqual(pri['modes'],pyw.devmodes(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devmodes,'bad0')

# test get cmds
class DevCMDsTestCase(CardTestCase):
    def test_devcmds(self):
        self.assertIsInstance(pyw.devmodes(self.card),list)
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devmodes,'bad0')

# test devinfo - the key-value pairs of devinfo are tested via others
class DevInfoTestCase(CardTestCase):
    def test_devinfobycard(self):
        self.assertIsInstance(pyw.devinfo(self.card),dict)
    def test_devinfobydev(self):
        self.assertIsInstance(pyw.devinfo(pri['dev']),dict)
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devinfo,'bad0')

# test phyinfo
class PhyInfoTestCase(CardTestCase):
    def test_phyinfo(self):
        self.assertIsInstance(pyw.phyinfo(self.card),dict)
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.phyinfo,'bad0')

# test txset
# currently txset is not supported by my cards

# test txget
class TXGetTestCase(CardTestCase):
    def test_txget(self):
        self.assertEquals(pri['tx'],pyw.txget(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.txget,'bad0')

# test chget/chset
# since we are using a non-associated card, we will get None for chget
# testing chset simulatenously allows us to test chset then chget
# NOTE: we don't test for specific ch in chget, just in the infitesimal chance
# that somehow the ch was reset etc
class CHGetSetTestCase(CardTestCase):
    def test_chsetget(self):
        pyw.down(self.card)
        pyw.modeset(self.card,'monitor')
        pyw.up(self.card)
        self.assertEqual(None,pyw.chset(self.card,1))
        self.assertIsInstance(pyw.chget(self.card),int)
        pyw.down(self.card)
        pyw.modeset(self.card,'managed')
        pyw.up(self.card)
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.chset,pri['dev'],1)
        self.assertRaises(error,pyw.chget,'bad0')
    def test_invalidcharg(self):
        self.assertRaises(error,pyw.chset,self.card,0,None)
    def test_invalidchwarg(self):
        self.assertRaises(error,pyw.chget,self.card,1,'HT30+')

# test freqset
# because freqset was already tested in chgetset, we only test invalid args
class FreqSetTestCase(CardTestCase):
    def test_invalidrfarg(self):
        # we test both an invalid RF and an RF the card does not support
        self.assertRaises(error,pyw.freqset,self.card,2410)
        self.assertRaises(error,pyw.freqset,self.card,4960)

# test modeget
class ModeGetTestCase(CardTestCase):
    def test_modeget(self):
        self.assertEquals('managed',pyw.modeget(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.modeget,'bad0')

# test modeset
class ModeSetTestCase(CardTestCase):
    def test_modeset(self):
        pyw.down(self.card)
        self.assertEquals(None,pyw.modeset(self.card,'monitor'))
        self.assertEquals(None,pyw.modeset(self.card,'managed'))
        pyw.up(self.card)
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.modeset,'bad0','monitor')
    def test_invalidmodearg(self):
        self.assertRaises(error,pyw.modeset,self.card,'foobar')
    def test_invalidmonitorflagarg(self):
        self.assertRaises(error,pyw.modeset,self.card,'monitor','bad')
        self.assertRaises(error,pyw.modeset,self.card,'managed','fcsfail')

# test ifaces
class IfacesTestCase(CardTestCase):
    def test_ifaces(self):
        self.assertIsInstance(pyw.ifaces(self.card),list)
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.ifaces,'bad0')

# test devadd/devdel
"""
class DevAddDelTestCase(CardTestCase):
    def test_devadddel(self):
        card = pyw.devadd(self.card,'test0','monitor')
        self.assertTrue(card.dev in pyw.winterfaces())
        pyw.devdel(card)
        self.assertFalse(card.dev in pyw.winterfaces())
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devadd,'bad0','test0','monitor')
        self.assertRaises(error,pyw.devdel,'bad0')
    def test_invalidmodearg(self):
        self.assertRaises(error,pyw.devadd,self.card,'test0','foobar')
    def test_invalidflagsarg(self):
        self.assertRaises(error,pyw.devadd,self.card,'test0','monitor','foobar')
        self.assertRaises(error,pyw.devadd,self.card,'test0','managed','fcsfail')

# test devset
class DevSetTestCase(CardTestCase):
    def test_devset(self):
        card = pyw.devset(self.card,'unittest0')
        self.assertTrue(pyw.iswireless('unittest0'))
        self.assertFalse(pyw.iswireless(pri['dev']))
        pyw.devset(card,pri['dev'])
        self.assertFalse(pyw.iswireless('unittest0'))
    def test_invalidcardarg(self):
        self.assertRaises(error,pyw.devset,'bad0','managed')
    def test_invalidndevarg(self):
        self.assertRaises(error,pyw.devset,self.card,None)
"""
class IsConnectedTestCase(CardTestCase):
    def test_isconnected(self):
        self.assertFalse(pyw.isconnected(self.card))
    def test_invalidcardarg(self):
        self.assertRaises(error, pyw.isconnected, 'bad0')

class PhyListTestCase(unittest.TestCase):
    def test_phylist(self):
        self.assertTrue((pri['phy'],'phy{0}'.format(pri['phy'])) in pyw.phylist())

class IfInfoTestCase(CardTestCase):
    def test_ifinfo(self):
        iinfo = pyw.ifinfo(self.card)
        self.assertTrue(pri['driver'] == iinfo['driver'])
        self.assertTrue(pri['chipset'] == iinfo['chipset'])
    def test_invalidcardarg(self):
        self.assertRaises(error, pyw.ifinfo, 'bad0')

def pyvers():
    return "{0}.{1}.{2}".format(sys.version_info.major,
                                sys.version_info.minor,
                                sys.version_info.micro)
if __name__ == '__main__':
    print("Testing PyRIC v{0} pyw v{1} on Python {2}".format(pyric.version,
                                                             pyw.__version__,
                                                             pyvers()))
    unittest.main()